﻿using System;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;
using Aquaforest.OCR.Api;
using Aquaforest.OCR.Definitions.Enums;
using System.Collections.Generic;

namespace ConvertFolderOfFiles
{
	public partial class Form1 : Form
	{
		FileInfo[] sourceFilesList;
		private string currentDocument;
		private Ocr ocr;
		private PreProcessor preProcessor;
		private int pageCompleted;
		private BackgroundWorker ocrProcessingBackgroundWorker;
		private string license = "";
		private int blankPageThreshold;
		private bool removeExistingPdfText;
		private bool tables;
		private bool noPictures;
		private int despeckle;
		private bool deskew;
		private bool autorotate;
		private bool enablePdfOutput;
		private bool enableRtfOutput;
		private bool enableTextOutput;
		private SupportedLanguages language;
		private bool removeBlanks;

		public Form1()
		{
			InitializeComponent();
			GetLanguageNames();
			comboBoxDespeckle.SelectedIndex = 0;
			checkBoxTxt.Checked = true;
		}

		private void GetLanguageNames()
		{
			foreach (string language in Enum.GetNames(typeof(SupportedLanguages)))
			{
				comboLanguages.Items.Add(language.Replace('_', ' '));
			}

			comboLanguages.SelectedIndex = 0;
		}

		private void GetSourceFileList()
		{
			DirectoryInfo directoryInfo = new DirectoryInfo(textBoxSource.Text);

			string[] extensions = new string[] { "*.bmp", "*.tif", "*.tiff", "*.pdf" };

			List<FileInfo> files = new List<FileInfo>();

			foreach (string ext in extensions)
			{
				files.AddRange(directoryInfo.GetFiles(ext));
			}

			sourceFilesList = new FileInfo[files.Count];
			sourceFilesList = (FileInfo[])files.ToArray();
		}

		private void buttonConvert_Click(object sender, EventArgs e)
		{
			GetOcrProcessorOptions();
			ConfigureOcrProcessor();
			ocr.StatusUpdate += OcrStatusUpdate;

			GetSourceFileList();
			if (sourceFilesList.Length > 0)
			{
				EnableUI(false);
				RunOcrOnBackgroundWorker();
			}
		}
		
		private void OcrStatusUpdate(object sender, Aquaforest.OCR.Definitions.StatusUpdateEventArgs pageCompleteEventArgs)
        {
            pageCompleted = pageCompleteEventArgs.PageNumber;
            ocrProcessingBackgroundWorker.ReportProgress(0);
        }

        private void RunOcrOnBackgroundWorker()
        {
            ocrProcessingBackgroundWorker = new BackgroundWorker();
            ocrProcessingBackgroundWorker.DoWork += DoWork;
            ocrProcessingBackgroundWorker.RunWorkerCompleted += OnOcrCompleted;
            ocrProcessingBackgroundWorker.WorkerReportsProgress = true;
            ocrProcessingBackgroundWorker.ProgressChanged += OnOcrProgressChanged;
            ocrProcessingBackgroundWorker.WorkerSupportsCancellation = true;
            ocrProcessingBackgroundWorker.RunWorkerAsync();
        }

        private void GetOcrProcessorOptions()
        {
            language = GetLanguageValueFromSelectedIndex();
            enableTextOutput = checkBoxTxt.Checked;
            enableRtfOutput = checkBoxRtf.Checked;
            enablePdfOutput = checkBoxPdf.Checked;
            autorotate = checkBoxAutorotate.Checked;
            deskew = checkBoxDeskew.Checked;
            despeckle = comboBoxDespeckle.SelectedIndex;
            noPictures = checkBoxNoPictures.Checked;
            tables = checkBoxTables.Checked;
            removeExistingPdfText = checkBoxRemoveExistingText.Checked;

            removeBlanks = checkBoxBlankPage.Checked;
            if (checkBoxBlankPage.Checked)
            {
                blankPageThreshold = Convert.ToInt32(textBoxBlank.Text);
            }
        }

        private void ConfigureOcrProcessor()
        {
            if (ocr != null)
            {
                ocr.Dispose();
            }

            ocr = new Ocr();
            ocr.License = license;
            string resourceFolder = Path.GetFullPath(@"..\..\..\..\..\bin\");
            string currentEnvironmentVariables = Environment.GetEnvironmentVariable("PATH");
            if (!currentEnvironmentVariables.Contains(resourceFolder))
            {
                Environment.SetEnvironmentVariable("PATH", currentEnvironmentVariables + ";" + resourceFolder);
            }
            ocr.ResourceFolder = resourceFolder;
            preProcessor = new PreProcessor();
            ocr.Language = language;
            ocr.EnableTextOutput = enableTextOutput;
            ocr.EnableRtfOutput = enableRtfOutput;
            ocr.EnablePdfOutput = enablePdfOutput;
            preProcessor.Autorotate = autorotate;
            preProcessor.Deskew = deskew;
            preProcessor.Despeckle = despeckle;
            preProcessor.NoPictures = noPictures;
            preProcessor.Tables = tables;
            ocr.RemoveExistingPDFText = removeExistingPdfText;

            if (removeBlanks)
            {
                preProcessor.BlankPageThreshold = blankPageThreshold;
            }
        }

        private void OnOcrCompleted(object sender, RunWorkerCompletedEventArgs runWorkerCompletedEventArgs)
        {
            labelProgress.Text = "";
            currentDocument = "";
            pageCompleted = 0;
            EnableUI(true);
        }

        private void OnOcrProgressChanged(object sender, ProgressChangedEventArgs progressChangedEventArgs)
        {
            labelProgress.Text = string.Format("Processing {0}: page {1} complete.", Path.GetFileName(currentDocument), pageCompleted);
        }

        private void DoWork(object sender, DoWorkEventArgs doWorkEventArgs)
        {
            foreach (FileInfo fileInfo in sourceFilesList)
            {
                if (ocrProcessingBackgroundWorker.CancellationPending)
                {
                    break;
                }

                currentDocument = fileInfo.FullName;
                ocrProcessingBackgroundWorker.ReportProgress(0);

                string extension = fileInfo.Extension.ToLower();

                if (extension.Contains(".bmp"))
                {
                    ocr.ReadBMPSource(fileInfo.FullName);
                }
                else if (extension.Contains(".tif") || extension.Contains(".tiff"))
                {
                    ocr.ReadTIFFSource(fileInfo.FullName);
                }
                else if (extension.Contains(".pdf"))
                {
                    ocr.ReadPDFSource(fileInfo.FullName);
                }

                ocr.Recognize(preProcessor);
                if (ocrProcessingBackgroundWorker.CancellationPending)
                {
                    break;
                }

                if (checkBoxPdf.Checked)
                {
                    ocr.SavePDFOutput(
                        Path.GetFullPath(textBoxDestination.Text) + Path.DirectorySeparatorChar +
                        Path.GetFileNameWithoutExtension(fileInfo.Name) + "_searchable.pdf", true);
                }

                if (checkBoxRtf.Checked)
                {
                    ocr.SaveRTFOutput(
                        Path.GetFullPath(textBoxDestination.Text) + Path.DirectorySeparatorChar +
                        Path.GetFileNameWithoutExtension(fileInfo.Name) + ".rtf", true);
                }

                if (checkBoxTxt.Checked)
                {
                    ocr.SaveTextOutput(
                        Path.GetFullPath(textBoxDestination.Text) + Path.DirectorySeparatorChar +
                        Path.GetFileNameWithoutExtension(fileInfo.Name) + ".txt", true);
                }
            }
        }

        private void buttonBrowseDestination_Click(object sender, EventArgs e)
        {
            textBoxDestination.Text = GetFolder();
        }

        private void buttonBrowseSource_Click(object sender, EventArgs e)
        {
            textBoxSource.Text = GetFolder();
        }

        private string GetFolder()
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.ShowDialog();
            return folderBrowserDialog.SelectedPath;
        }

        private SupportedLanguages GetLanguageValueFromSelectedIndex()
        {            
            Array languageses = Enum.GetValues(typeof (SupportedLanguages));
            SupportedLanguages language = (SupportedLanguages)languageses.GetValue(comboLanguages.SelectedIndex);
            return language;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            if (ocrProcessingBackgroundWorker != null)
            {
                ocrProcessingBackgroundWorker.CancelAsync();
                ocr.Abort();
                ocrProcessingBackgroundWorker.Dispose();
            }
        }

        private void EnableUI(bool enable)
        {
            textBoxSource.Enabled = enable;
            textBoxDestination.Enabled = enable;
            buttonConvert.Enabled = enable;
            buttonCancel.Enabled = !enable;
            buttonBrowseSource.Enabled = enable;
            buttonBrowseDestination.Enabled = enable;
            comboLanguages.Enabled = enable;
            comboBoxDespeckle.Enabled = enable;
            checkBoxTxt.Enabled = enable;
            checkBoxTables.Enabled = enable;
            checkBoxRtf.Enabled = enable;
            checkBoxPdf.Enabled = enable;
            checkBoxNoPictures.Enabled = enable;
            checkBoxDeskew.Enabled = enable;
            checkBoxAutorotate.Enabled = enable;
            checkBoxBlankPage.Enabled = enable;
            if (checkBoxBlankPage.Checked)
            {
                textBoxBlank.Enabled = enable;
            }
            else
            {
                textBoxBlank.Enabled = false;
            }
        }

        private void checkBoxPdf_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxPdf.Checked)
            {
                groupBoxPDFOutputOptions.Enabled = true;
            }
            else
            {
                groupBoxPDFOutputOptions.Enabled = false;
            }
        }

        private void checkBoxBlankPage_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxBlankPage.Checked)
            {
                textBoxBlank.Enabled = true;
            }
            else
            {
                textBoxBlank.Enabled = false;
            }
        }

        private void textBoxStart_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == '\b')
            {
                return;
            }

            if (e.KeyChar < '0' || e.KeyChar > '9')
            {
                e.Handled = true;
            }
        }

        private void textBoxDigits_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == '\b')
            {
                return;
            }

            if (e.KeyChar < '0' || e.KeyChar > '9')
            {
                e.Handled = true;
            }
        }

        private void textBoxBlank_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == '\b')
            {
                return;
            }

            int value;
            if (e.KeyChar < '0' || e.KeyChar > '9' || !Int32.TryParse(textBoxBlank.Text + e.KeyChar, out value))
            {
                e.Handled = true;
            }
        }
    }
}
