﻿using Aquaforest.OCR.Api;
using Aquaforest.OCR.Definitions;
using Aquaforest.PDF;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace ExtractText
{
    class Program
    {
        static Ocr ocr;

        static void Main(string[] args)
        {
            DirectoryInfo directory = new DirectoryInfo(@"C:\Aquaforest\OCRSDK\samples\documents\source");

            string outputDirectory = @"C:\Aquaforest\OCRSDK\samples\documents\output";
            if (!Directory.Exists(outputDirectory)) Directory.CreateDirectory(outputDirectory);

            foreach (var file in directory.GetFiles("*.pdf", SearchOption.AllDirectories))
            {
                Console.WriteLine("Processing PDF file: {0}", file.FullName);

                var documentPagesText = new Dictionary<int, string>();
                bool hasImageOnlyPage = CheckDocumentUsingToolkit(file.FullName, documentPagesText);

                if (hasImageOnlyPage)
                {
                    OcrPdf(file.FullName, documentPagesText);
                }

                if (documentPagesText.Count > 0)
                {
                    string output = Path.Combine(outputDirectory, Path.ChangeExtension(file.Name, "txt"));
                    WriteTextToFile(output, documentPagesText);
                }
                else
                {
                    Console.WriteLine("No text found.");
                }

                Console.WriteLine("");
            }
        }

        /// <summary>
        /// Check each page of source PDF document to see if it has any text. If there is text,
        /// store it in a dictionary or else set the hasImageOnlyPage flag to true.
        /// </summary>
        static bool CheckDocumentUsingToolkit(string source, Dictionary<int, string> documentPagesText)
        {
            PDFDocument doc = null;
            bool hasImageOnlyPage = false;

            try
            {
                doc = new PDFDocument(source);
                for (int i = 1; i <= doc.NumberOfPages; i++)
                {
                    string pageText = doc.GetText(i);
                    documentPagesText[i] = pageText;

                    if (!hasImageOnlyPage) // If the document has at least 1 image-only page, it will be OCRed
                    {
                        // Check if this page is image-only.
                        // ASSUMPTION: if the page doesn't have any text, it is considered as image-only,
                        // even if it is a blank page with no images
                        hasImageOnlyPage =
                            string.IsNullOrEmpty(pageText.Replace("\r\n", "").Replace("\n", "").Replace("\r", ""));
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error reading PDF file: {0}", e.Message);
            }
            finally
            {
                if (doc != null) doc.Close();
            }

            return hasImageOnlyPage;
        }


        static bool OcrPdf(string source,
            Dictionary<int, string> documentPagesText)
        {
            try
            {
                Console.WriteLine("OCRing...");

                using (ocr = new Ocr())
                {
                    string resourceFolder = @"C:\Aquaforest\OCRSDK\bin";

                    string currentEnvironmentVariables =
                    Environment.GetEnvironmentVariable("PATH");
                    if (!currentEnvironmentVariables.Contains(resourceFolder))
                    {
                        Environment.SetEnvironmentVariable("PATH",
                            currentEnvironmentVariables + ";" + resourceFolder);
                    }

                    ocr.ResourceFolder = resourceFolder;
                    ocr.EnableConsoleOutput = true;
                    ocr.Language = SupportedLanguages.English;
                    ocr.EnablePdfOutput = true;
                    ocr.StatusUpdate += (sender, pageCompleteEventArgs) =>
                        OcrStatusUpdate(sender, pageCompleteEventArgs, documentPagesText);

                    PreProcessor preProcessor = new PreProcessor();
                    preProcessor.Deskew = true;
                    preProcessor.Autorotate = false;

                    ocr.ReadPDFSource(source);

                    ocr.Recognize(preProcessor);

                    ocr.DeleteTemporaryFiles();

                    return true;
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Error in OCR Processing : {0}", e.Message);
                return false;
            }
        }

        static void OcrStatusUpdate(object sender,
            StatusUpdateEventArgs pageCompleteEventArgs,
            Dictionary<int, string> documentPagesText)
        {
            if (pageCompleteEventArgs.TextAvailable)
            {
                int pageNumber = pageCompleteEventArgs.PageNumber;
                string pageString = ocr.ReadPageString(pageNumber);

                // Optional
                string existingText;
                if (documentPagesText.TryGetValue(pageNumber, out existingText))
                {
                    if (!string.IsNullOrEmpty(existingText))
                    {
                        pageString += Environment.NewLine + existingText;
                    }
                }

                documentPagesText[pageNumber] = pageString;
            }
        }

        static void WriteTextToFile(string output,
            Dictionary<int, string> documentPagesText)
        {
            Console.WriteLine("Writing text to: {0}", output);

            for (int i = 1; i <= documentPagesText.Keys.Max(); i++)
            {
                string pageText;
                if (documentPagesText.TryGetValue(i, out pageText))
                {
                    File.AppendAllText(output, pageText + Environment.NewLine);
                }
            }
        }
    }
}