﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Aquaforest.OCR.Api;
using Aquaforest.OCR.Definitions;

namespace Zonal_OCR
{
	public partial class Form1 : Form
	{
		private Ocr ocr = new Ocr();
		private PreProcessor preProcessor = new PreProcessor();
		private int pageCompleted;
		private BackgroundWorker ocrProcessingBackgroundWorker;
		private Rectangle region = new Rectangle();
		private bool textAvailable;
		private bool imageAvailable;

		private string license = "";

		public Form1()
		{
			try
            {
                InitializeComponent();
                GetLanguageNames();
                comboBoxDespeckle.SelectedIndex = 0;
                ocr.License = license;

                string resourceFolder = Path.GetFullPath(@"..\..\..\..\..\bin\");
                string currentEnvironmentVariables = Environment.GetEnvironmentVariable("PATH");
                if (!currentEnvironmentVariables.Contains(resourceFolder))
                {
                    Environment.SetEnvironmentVariable("PATH", currentEnvironmentVariables + ";" + resourceFolder);
                }
                ocr.ResourceFolder = resourceFolder;

                string version = ocr.Version;
                buttonShowAllText.Enabled = false;
                buttonShowOcr.Enabled = false;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, "Aquaforest");
            }
		}

		private void GetLanguageNames()
		{
			foreach (string language in Enum.GetNames(typeof(SupportedLanguages)))
			{
				comboLanguages.Items.Add(language.Replace('_', ' '));
			}

			comboLanguages.SelectedIndex = 0;
		}

		private void buttonSource_Click(object sender, EventArgs e)
		{
			OpenFileDialog openFileDialog = new OpenFileDialog();
			openFileDialog.Filter = "PDF|*.pdf|TIFF|*.tif";
			openFileDialog.ShowDialog();
			if (File.Exists(openFileDialog.FileName))
			{
				textBoxSource.Text = openFileDialog.FileName;
				if (Path.GetExtension(openFileDialog.FileName).ToLower() == ".pdf")
				{
					ocr.ReadPDFSource(openFileDialog.FileName);
				}
				else
				{
					ocr.ReadTIFFSource(openFileDialog.FileName);
				}

				comboBoxPage.Items.Clear();
				for (int page = 1; page <= ocr.NumberPages; page++)
				{
					comboBoxPage.Items.Add(page);
					comboBoxPage.SelectedIndex = 0;
				}
			}
		}

		private void EnableUI(bool enable)
		{
			textBoxSource.Enabled = enable;
			buttonOcr.Enabled = enable;
			buttonSource.Enabled = enable;
			comboLanguages.Enabled = enable;
			comboBoxDespeckle.Enabled = enable;
			comboBoxPage.Enabled = enable;
			checkBoxTables.Enabled = enable;
			checkBoxNoPictures.Enabled = enable;
			checkBoxDeskew.Enabled = enable;
			checkBoxAutorotate.Enabled = enable;
			buttonShowAllText.Enabled = false;
			buttonShowOcr.Enabled = false;
		}

		private SupportedLanguages GetLanguageValueFromSelectedIndex()
		{
			Array languageses = Enum.GetValues(typeof(SupportedLanguages));
			SupportedLanguages language = (SupportedLanguages)languageses.GetValue(comboLanguages.SelectedIndex);
			return language;
		}

		private void RunOcrOnBackgroundWorker()
		{
			ocrProcessingBackgroundWorker = new BackgroundWorker();
			ocrProcessingBackgroundWorker.DoWork += DoWork;
			ocrProcessingBackgroundWorker.RunWorkerCompleted += OnOcrCompleted;
			ocrProcessingBackgroundWorker.WorkerReportsProgress = true;
			ocrProcessingBackgroundWorker.ProgressChanged += OnOcrProgressChanged;
			ocrProcessingBackgroundWorker.WorkerSupportsCancellation = true;
			ocrProcessingBackgroundWorker.RunWorkerAsync();
		}

		private void ConfigureOcrProcessor()
		{
			ocr.Language = GetLanguageValueFromSelectedIndex();
			preProcessor.Autorotate = checkBoxAutorotate.Checked;
			preProcessor.Deskew = checkBoxDeskew.Checked;
			preProcessor.Despeckle = comboBoxDespeckle.SelectedIndex;
			preProcessor.NoPictures = checkBoxNoPictures.Checked;
			preProcessor.Tables = checkBoxTables.Checked;
			preProcessor.RemoveLines = true;
		}

		private void OnOcrCompleted(object sender, RunWorkerCompletedEventArgs runWorkerCompletedEventArgs)
		{
			EnableUI(true);
			if (imageAvailable)
			{
				DisplayImage();
				SizeControlToFitImage();
			}

			buttonShowAllText.Enabled = textAvailable;
			buttonShowOcr.Enabled = textAvailable;
		}

		private void DisplayImage()
		{
			pictureBoxPage.Image = ocr.GetPageImage(ocr.StartPage);
			pictureBoxPage.Invalidate();
		}

		private void OnOcrProgressChanged(object sender, ProgressChangedEventArgs progressChangedEventArgs)
		{
		}

		private void DoWork(object sender, DoWorkEventArgs doWorkEventArgs)
		{
			ocr.StatusUpdate += OcrStatusUpdate;
			ocr.Recognize(preProcessor);
		}
		
		private void OcrStatusUpdate(object sender, StatusUpdateEventArgs pageCompleteEventArgs)
        {
            pageCompleted = pageCompleteEventArgs.PageNumber;
            ocrProcessingBackgroundWorker.ReportProgress(0);
            textAvailable = pageCompleteEventArgs.TextAvailable;
            imageAvailable = pageCompleteEventArgs.ImageAvailable;
        }

        private void buttonOcr_Click(object sender, EventArgs e)
        {
            ocr.StartPage = (int)comboBoxPage.SelectedItem;
            ocr.EndPage = (int)comboBoxPage.SelectedItem;
            ocr.EnablePdfOutput = true;
            ocr.EnableTextOutput = false;
            ConfigureOcrProcessor();
            if (File.Exists(textBoxSource.Text))
            {
                if (pictureBoxPage.Image != null)
                {
                    pictureBoxPage.Image.Dispose();
                }

                EnableUI(false);
                RunOcrOnBackgroundWorker();
            }
        }

        private void SizeControlToFitImage()
        {
            pictureBoxPage.Width = pictureBoxPage.Image.Width;
            pictureBoxPage.Height = pictureBoxPage.Image.Height;
        }

        private void pictureBoxPage_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                pictureBoxPage.Invalidate();
                region.X = e.X;
                region.Y = e.Y;
                region.Width = 0;
                region.Height = 0;
            }
        }

        private void pictureBoxPage_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                GetRegion(e);
            }
        }

        private void GetRegion(MouseEventArgs e)
        {
            if (e.X < region.X)
            {
                region.Width = region.X - e.X;
                region.X = e.X;
            }
            else
            {
                region.Width = e.X - region.X;
            }

            if (e.Y < region.Y)
            {
                region.Height = region.Y - e.Y;
                region.Y = e.Y;
            }
            else
            {
                region.Height = e.Y - region.Y;
            }

            pictureBoxPage.Invalidate();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
        }

        private void pictureBoxPage_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawRectangle(new Pen(Color.Red, 5), region);
        }

        private void pictureBoxPage_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                GetRegion(e);
                pictureBoxPage.Invalidate();
            }
        }

        private void buttonShowOcr_Click(object sender, EventArgs e)
        {
            Form2 ocrText = new Form2();
            ocrText.labelOcrText.Text = GetRegionalText();
            ocrText.panel1.Width = ocrText.labelOcrText.Width;
            ocrText.panel1.Height = ocrText.labelOcrText.Height;
            ocrText.ShowDialog();
        }

        private void buttonShowAllText_Click(object sender, EventArgs e)
        {
            Form2 ocrText = new Form2();
            ocrText.labelOcrText.Text = GetAllText();
            ocrText.panel1.Width = ocrText.labelOcrText.Width;
            ocrText.panel1.Height = ocrText.labelOcrText.Height;
            ocrText.ShowDialog();
        }

        private string GetRegionalText()
        {
            return ocr.ReadPageString((int) comboBoxPage.SelectedItem, region);
        }
         
        private string GetAllText()
        {
            return ocr.ReadPageString((int) comboBoxPage.SelectedItem);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (pictureBoxPage.Image != null)
            {
                pictureBoxPage.Image.Dispose();
            }
        }
    }
}
